

#include "clause.h"


void clause::insert( literal lit )
{
   for( std::list< literal > :: const_iterator
           p = repr. begin( );
           p != repr. end( );
           ++ p )
   {
      if( *p == lit )
         return;
   }
   repr. push_back( lit );
}


bool clause::subsumes( const clause& other ) const 
{
   for( std::list< literal > :: const_iterator
           p = repr. begin( );
           p != repr. end( );
           ++ p )
   {
      bool inother = false;
      for( std::list< literal > :: const_iterator
              q = other. repr. begin( );
              q != other. repr. end( ) && ! inother;
              ++ q )
      {
         if( *p == *q )
            inother = true;
      }
      
      if( ! inother )
         return false;
   }
   return true;
}


bool clause::operator == ( const clause& cl ) const
{
   return this -> subsumes( cl ) && cl. subsumes( *this );
}


bool clause::operator != ( const clause& cl ) const
{
   return ! this -> subsumes( cl ) || ! cl. subsumes( *this );
}



bool clause::istrue( const interpretation& i ) const
{
   for( std::list< literal > :: const_iterator 
           p = repr. begin( );
	   p != repr. end( );
	   ++ p )
   {
      if( i. isdefined( p -> at ))
      {
         bool val = i. value( p -> at );
	 if( p -> positive && val )           
            return true;                  
	 if( ! p -> positive && ! val )
            return true; 
      }
   }
   return false;
}



bool clause::isfalse( const interpretation& i ) const
{

   for( std::list< literal > :: const_iterator
           p = repr. begin( );
	   p != repr. end( );
	   ++ p )
   {
      if( i. isdefined( p -> at ))
      {
         bool val = i. value( p -> at );
	 if( p -> positive && val )
            return false;
	       // Positive literal for which I is true. 

	 if( ! p -> positive && ! val )
            return false;
	       // Negative literal for which I is false. 
      }
      else
         return false;
            // Unassigned atom. We don't know for sure that the clause 
	    // is false. 
   }
   return true; 
}



unsigned int clause::nrunassigned( const interpretation& i ) const
{
   unsigned int nr = 0;
   for( std::list< literal > :: const_iterator
           p = repr. begin( );
	   p != repr. end( );
	   ++ p )
   {
      if( ! i. isdefined( p -> at ))
         ++ nr;
   }
   return nr;
}


literal clause::findunassigned( const interpretation& i ) const
{
   for( std::list< literal > :: const_iterator
           p = repr. begin( );
	   p != repr. end( );
	   ++ p )
   {
      if( ! i. isdefined( p -> at ))
         return *p;
   }
   ASSERT( false );
   exit(0);
}


clause clause::resolve( atom at, const clause& apos, const clause& aneg )
{
   clause resolvent;
 
   literal poslit = literal::literal( true, at );
   literal neglit = literal::literal( false, at );

   for( std::list< literal > :: const_iterator
           p = apos. repr. begin( );
           p != apos. repr. end( );
           ++ p )
   {
      if( *p != poslit )
         resolvent. insert(*p);
   }

   for( std::list< literal > :: const_iterator
           p = aneg. repr. begin( );
           p != aneg. repr. end( );
           ++ p )
   {
      if( *p != neglit ) 
         resolvent. insert( *p );
   }

   return resolvent; 
}


std::ostream& operator << ( std::ostream& stream, const clause& cl )
{
   stream << "[ ";
   for( std::list< literal > :: const_iterator
           p = cl. repr. begin( );
           p != cl. repr. end( );
           ++ p )
   {
      if( p != cl. repr. begin( ))
         stream << ", ";

      stream << *p;
   }
   
   stream << " ]";
   return stream;
}


