

#ifndef CLAUSE_INCLUDED
#define CLAUSE_INCLUDED   1


#include <list>
#include <iostream>


#include "literal.h"
#include "interpretation.h"


class clause
{

   std::list< literal > repr;

public:
   void insert( literal lit );
      // Insert literal into the clause. 
      // Multiple occurrences are avoided.

   bool subsumes( const clause& other ) const;
      // true if we subsume other. 


   bool operator == ( const clause& cl ) const;
   bool operator != ( const clause& cl ) const;
      // Both are implemented through subsumes.

   unsigned int size( ) const { return repr. size( ); }

   bool istrue( const interpretation& i ) const;
   bool isfalse( const interpretation& i ) const;

   unsigned int nrunassigned( const interpretation& i ) const;
      // Returns number of atoms in cl that have no assignment in i. 

   literal findunassigned( const interpretation& i ) const;
      // Returns a (the first) unassigned literal. 


   static clause resolve( atom at, const clause& apos, const clause& aneg );
      // Constructs ( apos \ { at } ) union ( aneg \ { -at } ).

   friend std::ostream& operator << ( std::ostream& , const clause& cl );

};


std::ostream& operator << ( std::ostream& stream, const clause& cl );


#endif

