

#include "clauseset.h"


void clauseset::insert( const clause& cl )
{
   clauses. push_back( 
         clausederiv::clausederiv( cl, 
                                   std::list< unsigned int > :: list( )));
}


void clauseset::insert( const clause& cl,
	                unsigned int par1, unsigned int par2 )
{
   std::list< unsigned int > deriv;
   deriv. push_back(par1);
   deriv. push_back(par2);

   clauses. push_back( clausederiv::clausederiv( cl, deriv )); 
}


void clauseset::remove( const clause& cl )
{
   std::vector< clausederiv > :: iterator p = clauses. begin( );

   while( p != clauses. end( ))
   {
      if( p -> cl == cl )
      {
         clauses. erase(p);
	 remove(cl);   
	    // There may be more occurrences, and the vectors
	    // may have been corrupted due to the deletion. 
         return; 
      }
      ++ p;
   }
}


unsigned int clauseset::size( ) const
{
   return clauses. size( );
}


std::ostream& operator << ( std::ostream& stream, const clauseset& cl )
{
   stream  << "Clauseset:\n\n";
   for( unsigned int i = 0; i < cl. clauses. size( ); ++ i )
   {
      stream << i << " : ";
      if( cl. clauses [i]. deriv. size( ))
      {
         stream << "RES(" << cl. clauses [i]. deriv. front( ) << ",";
	 stream << cl. clauses [i]. deriv. back( ) << ")      ";
      }
      else
         stream << "              ";
      stream << cl. clauses [i]. cl << "\n";
   }
   stream << "\n\n";
   return stream;
}


namespace 
{

   // As long as c is a whitespace character, read a new character.
   // If we manage to read a non-white space character then true is returned.
   // Otherwise, we reached eof.

   bool skipwhite( std::istream& in, unsigned int& linecount, char& c )
   {
      while( c == '\n' || c == ' ' || c == '\t' )
      {
         if( c == '\n' )
            ++ linecount;
	 bool b = in. get(c);
	 if( !b )
            return false;
      }
      return true; 
   }


   // True if c is allowed in a (literal) name.

   bool allowedinname( char c )
   {
      if( c >= 'a' && c <= 'z' )
         return true;
      if( c >= 'A' && c <= 'Z' )
         return true;
      if( c >= '0' && c <= '9' )
         return true;
      if( c == '_' || c == '?' ) 
         return true;
      return false;
   }


   literal readliteral( std::istream& in, unsigned int linecount, 
		        char& lookahead ) throw( syntaxerror )
   {
      if( ! skipwhite( in, linecount, lookahead ))
         throw syntaxerror::syntaxerror( "expecting negation or atom name",
			                 linecount );

      bool polarity = true;
      if( lookahead == '-' )
      {
         polarity = false;
         if( ! in. get( lookahead ) || 
	     ! skipwhite( in, linecount, lookahead ))
            throw syntaxerror::syntaxerror( "expecting atom name",
		                            linecount ); 
      }

      if( ! allowedinname( lookahead ))
         throw syntaxerror::syntaxerror( "illegal character in literal name",
			                 linecount );
      std::string name; 
      name. push_back( lookahead ); 

      while( in. get( lookahead ) && allowedinname( lookahead ))
      {
         name. push_back( lookahead );
      }

      return literal::literal( polarity, atom::atom( name ));
   }

}


void clauseset::read( std::istream& in, unsigned int& linecount ) 
throw( syntaxerror )
{

   char lookahead = ' ';
   while( true )
   {
      if( ! skipwhite( in, linecount, lookahead ))
         return;

      if( lookahead != '[' )
         throw syntaxerror::syntaxerror( "expecting [", linecount );

      if( ! in. get( lookahead ) || ! skipwhite( in, linecount, lookahead ))
         throw syntaxerror::syntaxerror( 
			 "expecting literal instead of end of input", 
			 linecount );

      clause cl;
      if( lookahead != ']' )
      {
         cl. insert( readliteral( in, linecount, lookahead ));
	 if( ! skipwhite( in, linecount, lookahead ))
            throw syntaxerror::syntaxerror( 
			    "expecting ] or , instead of end of input",
			    linecount );
         while( lookahead != ']' )
         {
	    if( lookahead != ',' )
               throw syntaxerror::syntaxerror( "expecting to see a comma",
			                       linecount );

            if( ! in. get( lookahead ) || 
	        ! skipwhite( in, linecount, lookahead ))
               throw syntaxerror::syntaxerror( 
			     "expecting literal instead of end of input",
			     linecount );
	    cl. insert( readliteral( in, linecount, lookahead ));
	    if( ! skipwhite( in, linecount, lookahead ))
               throw syntaxerror::syntaxerror(
			      "expecting ] or , instead of end of input",
			      linecount );
         }
      }

      insert( cl ); 

      if( ! in. get( lookahead ))
         return;
      
   }

}

       


