

// Written by Hans de Nivelle,
// produces axioms that specify a sudoku.


#include <iostream>
#include <sstream>
#include <list>


struct point
{
   unsigned int x; 
   unsigned int y;

   point( unsigned int x, unsigned int y )
      :x(x), y(y)
   {
   }

};


std::string atom( unsigned int x,
		  unsigned int y,
		  unsigned int k )
{
   std::stringstream s;
   s << "val" << x << y << k;
   return s. str( );
}


// States that the set of points has at least one occurrence
// of k. 


void atleastone( std::ostream& stream, 
                 const std::list< point > & points, unsigned int k )
{
   stream << "[ ";
   for( std::list< point > :: const_iterator 
           p = points. begin( );
	   p != points. end( );
	   ++ p )
   {
      if( p != points. begin( ))
         stream << ", ";

      stream << atom( p -> x, p -> y, k ); 
   }
   stream << " ]\n";
}


// State that the set of points has at most one occurrence of k.

void atmostone( std::ostream& stream,
                const std::list< point > & points, unsigned int k )
{
   for( std::list< point > :: const_iterator
           p1 = points. begin( );
	   p1 != points. end( );
	   ++ p1 )
   {
      for( std::list< point  > :: const_iterator
              p2 = points. begin( );
	      p2 != p1;
	      ++ p2 )
      {
         stream << "[ ";
	 stream << "-" << atom( p1 -> x, p1 -> y, k ) << ", "; 
         stream << "-" << atom( p2 -> x, p2 -> y, k ) << " ]\n";
      }
   }
}


// State that (x,y) contains at least one k from 1..9. 

void atleastonedigit( std::ostream& stream, unsigned int x, unsigned int y )
{
   stream << "[ "; 
   for( unsigned int k = 1; k < 10; ++ k )
   {
      if( k != 1 )
         stream << ", ";
      stream << atom(x,y,k);
   }
   stream << " ]\n";
}


// State that (x,y) contains at most one k from 1..9.

void atmostonedigit( std::ostream& stream, unsigned int x, unsigned int y )
{
   for( unsigned int k1 = 1; k1 < 10; ++ k1 )
      for( unsigned int k2 = 1; k2 != k1; ++ k2 )
      {
         stream << "[ ";
	 stream << "-" << atom( x,y, k1 ) << ", ";
	 stream << "-" << atom( x,y, k2 ) << " ]\n";
      }
}


int main( unsigned int nrargs, char* ( args [] ))
{
   std::list< point > pppp;

   for( unsigned int d = 1; d < 10; ++ d )
   {
      // First we do the colums:

      for( unsigned int y = 1; y < 10; ++ y )
      {
         pppp. clear( ); 
	 for( unsigned int x = 1; x < 10; ++ x )
            pppp. push_back( point::point( x,y ));

	 atleastone( std::cout, pppp, d );
	 atmostone( std::cout, pppp, d );
      }

      // Then the rows:

      for( unsigned int x = 1; x < 10; ++ x ) 
      {
         pppp. clear( ); 
         for( unsigned int y = 1;  y < 10; ++ y )
            pppp. push_back( point::point( x,y ));

	 atleastone( std::cout, pppp, d );
	 atmostone( std::cout, pppp, d ); 
      }

      // The little blocks:

      for( unsigned int x1 = 1; x1 < 10; x1 += 3 )
         for( unsigned int y1 = 1; y1 < 10; y1 += 3 )
         {
            pppp. clear( );  
            for( unsigned int x2 = 0; x2 < 3; ++ x2 )
               for( unsigned int y2 = 0; y2 < 3; ++ y2 )
	          pppp. push_back( point::point( x1 + x2,  y1 + y2 ));

            atleastone( std::cout, pppp, d );
	    atmostone( std::cout, pppp, d );

         }

      std::cout << "\n\n";  
   }

   // No position should contain two numbers:

   for( unsigned int x = 1; x < 10; ++ x )
      for( unsigned int y = 1; y < 10; ++ y )
      {
         atleastonedigit( std::cout, x, y );
	 atmostonedigit( std::cout, x, y );
      }
}


